import java.util.*;
import java.io.*;

class BadFileFormat extends Exception{}

class Elettore{

   private String nome;
   private String cognome;
   private int pin;
   private boolean haVotato;

   public Elettore(String nome, String cognome, int pin){

      this.nome = nome;
      this.cognome = cognome;
      this.pin = pin;
      this.haVotato = false;
   }

   public void print(){
      System.out.println("Benvenuto/a "+nome+" "+cognome);
   }

   public boolean auth(int pin){
      
      if(!this.haVotato && this.pin == pin){
         this.haVotato = true;
         return true;
      }
      return false;
   }
}

public class Seggio{

   private int totSi, totNo, totVoid;
   private HashMap elettori = new HashMap();

   public Seggio(String fin) throws FileNotFoundException, IOException, BadFileFormat, NumberFormatException{
      BufferedReader in = new BufferedReader(new FileReader(fin));

      String line;

      while((line = in.readLine())!=null){

         StringTokenizer st = new StringTokenizer(line);

         if(st.countTokens()!=4)
            throw new BadFileFormat();

         String codiceFiscale = st.nextToken();
         String nome = st.nextToken();
         String cognome = st.nextToken();
         int pin = Integer.parseInt(st.nextToken()); //throws NumberFormatException

         elettori.put(codiceFiscale, new Elettore(nome,cognome,pin));
      }

      in.close();
   }

   private void votazione(){

      BufferedReader in = new BufferedReader(new InputStreamReader(System.in));

      try{
      
         System.out.print("Inserire codice fiscale: ");
         Elettore elettore = (Elettore)elettori.get(in.readLine());
         if(elettore==null){
            System.out.println("Lei non e' iscritto alla lista dei votanti");
            return;
         }

         int pin = 0;
         System.out.print("Inserire PIN: ");     
         try{
            pin = Integer.parseInt(in.readLine());
         }catch(NumberFormatException nfe){
            System.out.println("Il pin e' un numero. Ripetere la votazione");
         }
      
         if(!elettore.auth(pin)){
            System.out.println("Lei ha gia' votato, oppure il PIN e' errato");
            return;
         }

         System.out.print("Inserire la preferenza (1=si, 2=no, 3=bianca): ");
         try{
           int preferenza = Integer.parseInt(in.readLine());
           if(preferenza==1) totSi++;
           else if (preferenza==2) totNo++;
           else totVoid++;
         }catch(NumberFormatException nfe){
            totVoid++;
         }
      }catch(IOException iex){}
   }

   private void save(){

      try{
         PrintStream out = new PrintStream(new FileOutputStream("risultati.txt"));
        
         out.println("Aventi diritto: "+elettori.size());
         out.println("Votanti: "+(totSi+totNo+totVoid));
         out.println("Si: "+totSi);
         out.println("No: "+totNo);
         out.println("Bianche: "+totVoid);
      }catch(FileNotFoundException e){
         System.out.println(e);
      }
   }
   
   public static void main(String args[]){

      if(args.length!=1){
         System.out.println("Usage: java Seggio <file>");
         System.exit(-1);
      }

      Seggio seggio = null;
      
      try{
         seggio = new Seggio(args[0]);
      }catch(Exception e){
         System.out.println("Inizializzazione Seggio fallitai: "+e);
         System.exit(-2);
      }

      BufferedReader in = new BufferedReader(new InputStreamReader(System.in));

      boolean loop = true;

      while(loop){

         System.out.println("1) Vota 2) Esci");

         int choice = 0;
         try{
            choice = Integer.parseInt(in.readLine());
         }catch(NumberFormatException nfe){ //vai a default
         }catch(IOException ioe){} //vai a default

         switch(choice){
            case 1:
               seggio.votazione();
               break;
            case 2:
               seggio.save();
               loop = false;
               break;
            default:
               break; //continua il ciclo dall'inizio
         }
      }
   }
}        
